;/*
;#########################################################################
;#########################################################################
;#########################################################################
;#                                                               
;#   COPYRIGHT (c) 1998 by TransNexus, LLC                          
;#                                                                    
;#   This software contains proprietary and confidential information  
;#   of TransNexus, LLC. Except as may be set forth in the license    
;#   agreement under which this software is supplied, use, disclosure, 
;#   or reproduction is prohibited without the prior, express, written
;#   consent of TransNexus, LLC.                                      
;#                                     
;#
;# $Author: dmitry $
;#
;# $Date: 2004/01/26 13:36:40 $
;#
;# $Header: /home/scm/dev/cvsroot/src/osptoolkit/src/osp.def,v 1.2 2004/01/26 13:36:40 dmitry Exp $
;#
;# $Id: osp.def,v 1.2 2004/01/26 13:36:40 dmitry Exp $
;#
;# $Locker:  $
;#
;# $Name: TK-3_2_0-20041104 $
;#
;# $RCSfile: osp.def,v $
;#
;# $Revision: 1.2 $
;#
;# $Source: /home/scm/dev/cvsroot/src/osptoolkit/src/osp.def,v $
;#
;# $State: Exp $
;#
;# $Log: osp.def,v $
;# Revision 1.2  2004/01/26 13:36:40  dmitry
;# Added two new API calls
;#
;# Revision 1.1  2002/02/22 14:16:14  dmitry
;# Moved and restructured OSP Toolkit files from "osp" to "osptoolkit".
;#
;# Revision 1.19  1998/10/30 19:46:13  maria
;# fixes for tnlog.c
;#
;# Revision 1.18  1998/10/29 19:21:11  maria
;# minor fixes
;#
;# Revision 1.17  1998/10/23 19:05:06  maria
;# added funcs to support xmltest
;#
;# Revision 1.16  1998/10/23 16:55:20  maria
;# added functions to work w/xmltest
;#
;# Revision 1.15  1998/10/01 19:29:10  maria
;# Added OSPPTransactionRecordFailure
;#
;# Revision 1.14  1998/09/29 18:31:41  maria
;# added OSPPTransactionReInitializeAtDevice
;#
;# Revision 1.13  1998/09/25 22:02:03  maria
;# Added TransactionInitialize
;#
;# Revision 1.12  1998/09/17 14:39:24  maria
;# added list exports
;#
;# Revision 1.11  1998/09/17 14:30:43  maria
;# added callid delete
;#
;# Revision 1.10  1998/09/14 21:33:47  rvaughn
;# Added BuildTok for osptestsrvr
;#
;# Revision 1.9  1998/09/13 15:30:55  rvaughn
;# Fixes
;#
;# Revision 1.8  1998/09/11 20:35:59  rvaughn
;# Fixes to AuthRsp
;#
;# Revision 1.7  1998/08/04 18:57:32  rvaughn
;# format changes
;#
;# Revision 1.6  1998/07/29 01:33:58  rvaughn
;# signed/unsigned fixes. Added XML API stuff to osp.def temporarily
;# so that xmltest could link to the osp20.dll
;#
;# Revision 1.5  1998/07/20 14:19:50  rvaughn
;# OSPPCleanup
;#
;# Revision 1.4  1998/07/20 14:17:22  rvaughn
;# added OSPPCleanup
;#
;# Revision 1.3  1998/07/13 23:18:15  rvaughn
;# begin to remove stubs
;#
;# Revision 1.2  1998/07/02 12:39:55  maria
;# Fixed header.
;#
;# Revision 1.1  1998/07/02 12:36:32  maria
;# Initial version.
;#
;#
;##########################################################################
;##########################################################################
;##########################################################################
;*/

;/*
; * osp.def - Export file for osp.dll.
; */
LIBRARY      "OSP20"
DESCRIPTION  'OSP Windows Dynamic Link Library'

EXPORTS
    ; Explicit exports can go here
	OSPPInit
	OSPPCleanup
	OSPPProviderDelete
	OSPPProviderGetAuthorityCertificates
	OSPPProviderGetHTTPMaxConnections
	OSPPProviderGetHTTPPersistence
	OSPPProviderGetHTTPRetryDelay
	OSPPProviderGetHTTPRetryLimit
	OSPPProviderGetHTTPTimeout
	OSPPProviderGetLocalKeys
	OSPPProviderGetLocalValidation
	OSPPProviderGetNumberOfAuthorityCertificates
	OSPPProviderGetNumberOfServicePoints
	OSPPProviderGetServicePoints
	OSPPProviderGetSSLLifetime
	OSPPProviderNew
	OSPPProviderSetAuthorityCertificates
	OSPPProviderSetCapabilitiesURLs
	OSPPProviderSetHTTPMaxConnections
	OSPPProviderSetHTTPPersistence
	OSPPProviderSetHTTPRetryDelay
	OSPPProviderSetHTTPRetryLimit
	OSPPProviderSetHTTPTimeout
	OSPPProviderSetLocalKeys
	OSPPProviderSetLocalValidation
	OSPPProviderSetServicePoints
	OSPPProviderSetSSLLifetime
	OSPPTransactionAccumulateOneWayDelay
	OSPPTransactionAccumulateRoundTripDelay
	OSPPTransactionDelete
	OSPPTransactionGetFirstDestination
	OSPPTransactionGetNextDestination
    OSPPTransactionIndicateCapabilities
    OSPPTransactionInitializeAtDevice
    OSPPTransactionRecordFailure
    OSPPTransactionReinitializeAtDevice
	OSPPTransactionNew
	OSPPTransactionReportUsage
	OSPPTransactionRequestAuthorisation
	OSPPTransactionValidateAuthorisation
	OSPPAuthCnfFromElement
	OSPPAuthCnfToElement
	OSPPAuthCnfNew
	OSPPUsageCnfFromElement
	OSPPUsageCnfToElement
	OSPPUsageCnfNew
	OSPPAuthIndFromElement
	OSPPAuthIndToElement
	OSPPAuthIndNew
	OSPPStatusDelete
	OSPPAuthRspFromElement
	OSPPAuthRspToElement
	OSPPStatusNew
	OSPPAuthRspNew
	OSPPUsageIndFromElement
	OSPPUsageIndToElement
	OSPPUsageIndNew
	OSPPAuthReqFromElement
	OSPPXMLDocParse
	OSPPXMLDocCreate
	OSPPBfrNew
	OSPPAuthReqToElement
	OSPPCallIdNew
    OSPPCallIdDelete
	OSPPAuthReqNew
	OSPPAuthRspDelete
	OSPPDestNew
	OSPPUsageIndDelete
	OSPPAuthReqDelete
	OSPPAuthRspAddDest
	OSPPDestSetCallId
	OSPPAuthRspSetStatus
	OSPPStatusSetDesc
	OSPPUsageCnfSetStatus
	OSPPMimeHeaderBuild
	OSPPMimeMessageCreate
	OSPPXMLMessageCreate
	OSPPXMLMessageParse
	OSPPMimeMessageParse
	OSPPListAppend
	OSPPTokenNew
	OSPPListRemove
	OSPPListNext
	OSPPListFirst
	OSPPListNew
    OSPPListEmpty
	OSPPAuthRspSetTNDelayLimit
	OSPPAuthRspSetTNDelayPref
	OSPPAuthReqGetCallId
    OSPPDestGetCallIdSize
    OSPPDestGetCallIdValue
	OSPPDestAddToken
	OSPPDestSetLimit
	OSPPDestSetAuthority
	OSPPDestSetValidUntil
	OSPPDestSetAddr
	OSPPDestSetNumber
	OSPPAuthRspSetTrxId
	OSPPStatusSetCode
	OSPPAuthRspSetTimestamp
	OSPPAuthRspSetStatusDesc
	OSPPAuthRspSetStatusCode
	OSPPUsageCnfSetTimestamp
	OSPPDestSetValidAfter
	OSPPDestFirstToken
	OSPPListLast
	OSPPUsageCnfDelete
	OSPPTokenDelete
    OSPPTokenToElement
    OSPPTokenInfoDelete
    OSPPTokenInfoFromElement
    OSPPTokenInfoToElement
    OSPPTokenInfoSetTrxId
    OSPPTokenInfoSetValidUntil
    OSPPTokenInfoSetValidAfter
    OSPPTokenInfoSetDestNumber
    OSPPTokenInfoSetSourceNumber
    OSPPTokenInfoNew
    OSPPAuthCnfDelete
    OSPPAuthCnfSetValidUntil
    OSPPAuthCnfSetValidAfter
    OSPPAuthCnfSetStatus
    OSPPAuthCnfSetTimestamp
    OSPPAuthIndDelete
    OSPPAuthIndSetTimestamp
    OSPPUsageIndSetTNFailReason
    OSPPUsageIndSetTNDeviceId
    OSPPUsageIndSetTNCustId
    OSPPUsageIndSetDuration
    OSPPUsageIndAddSourceAlt
    OSPPUsageIndAddDestinationAlt
    OSPPAltInfoNew
    OSPPUsageIndSetDestNumber
    OSPPUsageIndSetSourceNumber
    OSPPUsageIndSetCallId
    OSPPUsageIndSetTransactionId
    OSPPUsageIndSetRole
    OSPPUsageIndSetTimestamp
    OSPPBfrDelete
    OSPPXMLElemDelete
    OSPPBfrSize
    OSPPAuthReqSetTNDeviceId
    OSPPAuthReqSetTNCustId
    OSPPAuthReqSetMaxDest
    OSPPAuthReqSetDestNumber
    OSPPAuthReqSetSourceNumber
    OSPPAuthReqSetTimestamp
    OSPPAuthReqAddCallIdValue
    OSPPTokenInfoSetCallId
    OSPPAuthIndAddDestinationAlt
    OSPPAuthIndAddSourceAlt
    OSPPAuthIndSetDestNumber
    OSPPAuthIndSetSourceNumber
    OSPPAuthIndSetCallId
    OSPPAuthIndSetRole
    OSPPAuthIndAddToken
    OSPPSSLWrapCleanup
    OSPPSecGetSSLLifetime
    OSPPSSLWrapInit
    OSPPSecSetSSLLifetime
    OSPPPKCS1PrivateKeyCopy
    OSPPPKCS1PrivateKeyCreate
    OSPPPKCS1PrivateKeyDelete
    OSPPSSLWrapSessionContextDelete
    OSPPSSLWrapSessionContextNew
    OSPPSSLWrapSessionContextValidate
    OSPPSSLWrapHandshake
    OSPPSSLWrapAttachConnection
    OSPPSSLWrapGetData
    OSPPSSLWrapSendData

