/**************************************************************************
*** COPYRIGHT (c) 2002 by TransNexus, Inc.                              ***
***                                                                     ***
*** This software is property of TransNexus, Inc.                       ***
*** This software is freely available under license from TransNexus.    ***
*** The license terms and conditions for free use of this software by   ***
*** third parties are defined in the OSP Toolkit Software License       ***
*** Agreement (LICENSE.txt).  Any use of this software by third         ***
*** parties, which does not comply with the terms and conditions of the ***
*** OSP Toolkit Software License Agreement is prohibited without        ***
*** the prior, express, written consent of TransNexus, Inc.             ***
***                                                                     ***
*** Thank you for using the OSP ToolKit(TM).  Please report any bugs,   ***
*** suggestions or feedback to support@transnexus.com                   ***
***                                                                     ***
**************************************************************************/








/*
 * osp.h - Universal OSP includes.
 */
#ifndef _OSP_H
#define _OSP_H

/*
 * internal SDK includes
 */

#include "osp/ospdatatypes.h"
#include "osp/ospinit.h"
#include "osp/osperrno.h"
#include "osp/ospossys.h"

/*
 * exposed OSP API prototypes and constants
 */

#include "osp/ospproviderapi.h"
#include "osp/osptransapi.h"
#include "osp/osplibversion.h"

#endif
